"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const discord_js_rate_limiter_1 = require("discord.js-rate-limiter");
const RegisterFunc_1 = __importDefault(require("../helper/RegisterFunc"));
const Command_1 = __importDefault(require("./Command"));
const Help_1 = __importDefault(require("../includes/Help"));
const defaultOptions = {
    defaultCategory: {
        name: "general",
        title: "General",
    },
};
class default_1 {
    /**
     * @param {Client} client The default discord.js client
     * @param {string} path The path relative to the root of the project
     * @param {
     * 	{
     * 		prefix: string,
     * 		prefixOnMention?: boolean,
     * 		customizablePrefix?: CustomizablePrefix,
     *  	defaultCategory?: Category,
     *  	help?: HelpOptions,
     *  	rateLimiter?: RateLimiterOptions
     * 	}
     * }options Creation options
     */
    constructor(client, path, options) {
        this.customizablePrefix = null;
        this.rateLimiter = null;
        const { prefix, prefixOnMention, customizablePrefix, defaultCategory, help, rateLimiter, } = options;
        this.client = client;
        this.prefix = prefix;
        this.prefixOnMention = prefixOnMention;
        if (customizablePrefix) {
            const { callback, options: prefixOptions } = customizablePrefix;
            this.customizablePrefix = {
                callback: callback ? callback : () => { },
                options: prefixOptions ? prefixOptions : {},
            };
        }
        if (rateLimiter) {
            const { enabled, amount, interval } = rateLimiter;
            // Make a discord.js rateLimiter object if rateLimiter should be enabled
            this.rateLimiter = enabled
                ? new discord_js_rate_limiter_1.RateLimiter(amount ? amount : 1, interval ? interval : 5000)
                : null;
        }
        // Add functions into the client object
        RegisterFunc_1.default(client);
        // Calculate the default category
        const defCategory = defaultCategory
            ? defaultCategory
            : defaultOptions.defaultCategory;
        if (defaultCategory) {
            // Default category name if not specified
            defCategory.name = defaultCategory.name
                ? defaultCategory.name
                : defaultOptions.defaultCategory.name;
            // Default category title if not specified
            defCategory.title = defaultCategory.title
                ? defaultCategory.title
                : defaultOptions.defaultCategory.title;
        }
        // Set the the first category
        this.categories = [defCategory];
        // Load all commands in the commands array
        this.commands = loadCommands(path);
        if (help) {
            const { include, category } = help;
            // If we should include a help command
            if (include) {
                // Load the default help command
                /**
                 * @type{
                 *     {
                 *       name: string,
                 *       invoke: []string,
                 *       description: string,
                 *       usage: string,
                 *       execute: function():Promise<void>,
                 *       category: string,
                 *       permissions: []string,
                 *     }
                 * }
                 */
                const helpCommand = Help_1.default;
                // Set help commands default category
                helpCommand.category = category ? category : defCategory.name;
                // Add the help command to the list
                this.commands.push(helpCommand);
            }
        }
    }
    /**
     * Adds another category to the list
     *
     * @param {string} name The name that will be referenced in commands
     * @param {string} title The title to display in the help message
     */
    addCategory(name, title) {
        this.categories.push({ name, title });
        return this;
    }
    messageEvent(message) {
        var _a, _b, _c, _d;
        return __awaiter(this, void 0, void 0, function* () {
            const client = message.client;
            if (!message.guild)
                return;
            // Dont do anything if the message author is a bot
            if (message.author.bot)
                return;
            let usePrefix = this.prefix;
            if (this.customizablePrefix) {
                const functionReturn = this.customizablePrefix.callback(message, this.customizablePrefix.options);
                usePrefix = functionReturn ? functionReturn : this.prefix;
            }
            // Dont do anything if the message does not start with the prefix or a mention
            if (!message.content.startsWith(usePrefix) &&
                !message.content.startsWith(`<@${(_a = this.client.user) === null || _a === void 0 ? void 0 : _a.id}>`) &&
                !message.content.startsWith(`<@!${(_b = this.client.user) === null || _b === void 0 ? void 0 : _b.id}>`))
                return;
            if (this.rateLimiter && this.rateLimiter.take(message.author.id)) {
                return message.channel.send("You're doing that do often, please try again later!");
            }
            if (this.prefixOnMention &&
                (message.content.trim() === `<@${(_c = this.client.user) === null || _c === void 0 ? void 0 : _c.id}> ` ||
                    message.content.trim() === `<@!${(_d = this.client.user) === null || _d === void 0 ? void 0 : _d.id}>`))
                yield message.channel.send(`The current prefix is \`${usePrefix}\``);
            // Filter all commands based on the permissions specified and the user's permissions
            const viableCommands = yield client.djsCommandControl.FilterCommands.byPermission(this.commands, message);
            // Check the message for every command's invoke
            viableCommands.forEach((command) => {
                command.invoke.forEach((invoke) => {
                    var _a, _b;
                    if (message.content.split(" ")[0] === `${usePrefix}${invoke}` ||
                        message.content.startsWith(`<@${(_a = client.user) === null || _a === void 0 ? void 0 : _a.id}> ${invoke}`) ||
                        message.content.startsWith(`<@!${(_b = client.user) === null || _b === void 0 ? void 0 : _b.id}> ${invoke}`)) {
                        message.react("👍");
                        command.execute(message, {
                            prefix: usePrefix,
                            commands: this.commands,
                            commandCategories: this.categories,
                        });
                    }
                });
            });
        });
    }
}
exports.default = default_1;
/**
 * Recursively load all files in directories and subdirectories
 * https://coderrocketfuel.com/article/recursively-list-all-the-files-in-a-directory-using-node-js
 *
 * @param {string} dirPath
 * @param {string[]} arrayOfFiles
 * @returns {string[]}
 */
function getAllFiles(dirPath, arrayOfFiles) {
    const files = fs_1.default.readdirSync(dirPath);
    arrayOfFiles = arrayOfFiles || [];
    files.forEach(function (file) {
        if (fs_1.default.statSync(dirPath + "/" + file).isDirectory()) {
            arrayOfFiles = getAllFiles(dirPath + "/" + file, arrayOfFiles);
        }
        else {
            arrayOfFiles.push(path_1.default.join(dirPath, "/", file));
        }
    });
    return arrayOfFiles;
}
/**
 * Loads all the commands
 * @recursive
 * @param {string} loadPath The path to load
 * @returns {{name: string, invoke: *[], string, description: string, usage: string, execute: (function(): Promise<void>), category: string, permissions: *[]}[]} Array of commands
 */
function loadCommands(loadPath) {
    /**
     * @type{[
     *     {
     *       name: string,
     *       invoke: []string,
     *       description: string,
     *       usage: string,
     *       execute: function():Promise<void>,
     *       category: string,
     *       permissions: []string,
     *     }
     * ]}
     */
    const commands = [];
    // Read all files in commands folder
    let foundFiles = getAllFiles(`${process.cwd()}/${loadPath}`, []);
    // Filter only javascript files in commands folder
    foundFiles.filter((file) => file.endsWith(".js"));
    foundFiles.forEach((file) => {
        // Test if the command has the correct exports
        let command = require(`${file}`);
        if (command instanceof Command_1.default) {
            command = command.command;
        }
        // If all this is true it is a valid commands
        if (command.name && command.invoke && command.execute) {
            // Add command to the list, but only use required exports
            commands.push({
                name: command.name,
                invoke: command.invoke,
                description: command.description,
                usage: command.usage,
                execute: command.execute,
                category: command.category ? command.category : "",
                permissions: command.permissions ? command.permissions : [],
                omitHelp: command.omitHelp ? command.omitHelp : false,
            });
        }
    });
    return commands;
}
